//
//  BFVoiceRecordManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/11/24.
//

import BFCommonKit
import BFMediaKit
import Foundation

class BFVoiceRecordManager: NSObject {
    // 录音相关
    var audioRecorder: BFRecorderManager?
    // 录音结果回调
    var endRecordHandle: ((PQVoiceModel?, Error?) -> Void)?
    var recordHandle: ((PQVoiceModel?, Error?) -> Void)?
    var cancelRecordHandle: ((PQVoiceModel?) -> Void)?

    var recorderFilePath: String = ""
    var beginRecordTime: Date = Date()
    var voiceModel: PQVoiceModel?
    ///字幕的回调 参数1： 字幕数据 ，参数2 ：对应的录音文件
    var subtitleRecordHandle: ((String?,String?) -> Void)?
    //进度回调
 
    var recorderProgrossHandle: ((Float64?) -> Void)?
    
    //停止是否为取消操作
    var isCancel:Bool = false

    /// 初始化方法
    /// - Parameters:
    ///   - token: NLS
    ///   - appid: NLS
    public init(token: String, appid: String) {
        super.init()
        audioRecorder = BFRecorderManager(token, appid: appid)
        audioRecorder?.delegate = self
    }

    /// 录制音频。 index初衷是记录录音顺序，废弃了
    func startRecord(index: Int) {
        recorderFilePath = exportAudiosDirectory

        if !directoryIsExists(dicPath: recorderFilePath) {
            BFLog(message: "文件夹不存在 \(recorderFilePath)")
            createDirectory(path: recorderFilePath)
        }
        recorderFilePath.append("recorder_\(index)_\(Date().timeIntervalSince1970).wav")
        BFLog(1, message: "开始录音 \(recorderFilePath)")

        BFLog(1, message: "开始录制")
        audioRecorder?.startRecord(recorderFilePath)
        beginRecordTime = Date()
    }

    /// 取消音频录制
    func cancleRecord() {
        stopRecord(cancel: true)
        
    }

    /// 结束音频录制
    func endRecord() {
        stopRecord(cancel: false)
    }

    /// 停止录制 1，正常停止 2，取消停止
    /// - Parameter isCancel: 是否为取消
     func stopRecord(cancel: Bool) {
//        if !(audioRecorder?.voiceRecorder.isStarted() ?? false) {
//            BFLog(message: "不是录制状态")
//            return
//        }
        isCancel = cancel
        audioRecorder?.stopRecord()
    }
}

// MARK: - 录音机回调

extension BFVoiceRecordManager: BFRecorderManagerDelegate {
    public func recorderProgress(_: BFRecorderManager, recoderTime: Double) {
        BFLog(message: "录音机进度:\(recoderTime)")
        recorderProgrossHandle?(recoderTime)
    }

    public func recorderDidStop(_ outfile: String) {
        let duration = Date().timeIntervalSince(beginRecordTime)
        if duration > 1 {
            
            if(isCancel){
                // 删除录制的原文件
                do {
                    try FileManager.default.removeItem(atPath: outfile)
                    print("取消后删除文件Success to remove recorder file. \(outfile)")
                } catch {
                    print("取消后删除文件Failed to remove recorder file. \(outfile)")
                }
                cancelRecordHandle?(nil)
            }else{
                
                // 处理降噪
                let noiseFilePath = outfile.replacingOccurrences(of: ".wav", with: "_noise.wav")
                BFLog(1, message: "降噪后地址：\(noiseFilePath) 原地址：\(outfile)")
                NXNoiseReduction().denoise(outfile, outFile: noiseFilePath)
                if let model = voiceModel {
                    model.wavFilePath = outfile
                    model.duration = "\(duration)"
                    endRecordHandle?(model, nil)
                }

    //            // 删除录制的原文件
    //            do {
    //                try FileManager.default.removeItem(atPath: outfile)
    //                print("Success to remove recorder file. \(outfile)")
    //            } catch {
    //                print("Failed to remove recorder file. \(outfile)")
    //            }
            }

        } else {
            cShowHUB(superView: nil, msg: "说话时间太短")
            cancelRecordHandle?(voiceModel)
        }
    }
    
    public func eventCallback(_: BFRecorderManager, asrResult: String,audioFilePath:String) {
        subtitleRecordHandle?(asrResult, audioFilePath)
    }
}
